@@ -1518,11 +1518,38 @@ tp_init_slots(struct tp_dispatch *tp,
 	return 0;
 }
 
+static uint32_t
+tp_accel_config_get_profiles(struct libinput_device *libinput_device)
+{
+	return LIBINPUT_CONFIG_ACCEL_PROFILE_NONE;
+}
+
+static enum libinput_config_status
+tp_accel_config_set_profile(struct libinput_device *libinput_device,
+			    enum libinput_config_accel_profile profile)
+{
+	return LIBINPUT_CONFIG_STATUS_UNSUPPORTED;
+}
+
+static enum libinput_config_accel_profile
+tp_accel_config_get_profile(struct libinput_device *libinput_device)
+{
+	return LIBINPUT_CONFIG_ACCEL_PROFILE_NONE;
+}
+
+static enum libinput_config_accel_profile
+tp_accel_config_get_default_profile(struct libinput_device *libinput_device)
+{
+	return LIBINPUT_CONFIG_ACCEL_PROFILE_NONE;
+}
+
 static int
 tp_init_accel(struct tp_dispatch *tp, double diagonal)
 {
+	struct evdev_device *device = tp->device;
 	int res_x, res_y;
 	struct motion_filter *filter;
+	int rc;
 
 	res_x = tp->device->abs.absinfo_x->resolution;
 	res_y = tp->device->abs.absinfo_y->resolution;
@@ -1545,7 +1572,18 @@ tp_init_accel(struct tp_dispatch *tp, double diagonal)
 	if (!filter)
 		return -1;
 
-	return evdev_device_init_pointer_acceleration(tp->device, filter);
+	rc = evdev_device_init_pointer_acceleration(tp->device, filter);
+	if (rc != 0)
+		return rc;
+
+	/* we override the profile hooks for accel configuration with hooks
+	 * that don't allow selection of profiles */
+	device->pointer.config.get_profiles = tp_accel_config_get_profiles;
+	device->pointer.config.set_profile = tp_accel_config_set_profile;
+	device->pointer.config.get_profile = tp_accel_config_get_profile;
+	device->pointer.config.get_default_profile = tp_accel_config_get_default_profile;
+
+	return 0;
 }
 
 static uint32_t
